<?php

namespace Drupal\erf_commerce\Plugin\EntityReferenceSelection;

use Drupal\Core\Entity\EntityReferenceSelection\SelectionPluginBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_product\Entity\ProductInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implementation of the VariationSelection Entity Reference Selection plugin.
 *
 * @EntityReferenceSelection(
 *   id = "erf_commerce_variation",
 *   label = @Translation("ERF Commerce: Product Variations for Product from URL"),
 *   group = "erf_commerce_variation",
 *   weight = 0
 * )
 */
class VariationSelection extends SelectionPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current route match service.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $routeMatch;

  /**
   * Constructs a new DefaultSelection object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $current_route_match
   *   The current route match service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, CurrentRouteMatch $current_route_match) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->entityTypeManager = $entity_type_manager;
    $this->routeMatch = $current_route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['erf_commerce_variation']['help'] = [
      '#markup' => '<p>' . $this->t('Available variations will be loaded for a product detected from the URL, either via a commerce product parameter (e.g. /product/{commerce_product}) or from the source entity of a registration.') . '</p>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getReferenceableEntities($match = NULL, $match_operator = 'CONTAINS', $limit = 0) {
    return $this->getProductVariationsForRoute();
  }

  /**
   * {@inheritdoc}
   */
  public function countReferenceableEntities($match = NULL, $match_operator = 'CONTAINS') {
    $valid_product_variation_ids = [];

    foreach ($this->getProductVariationsForRoute() as $bundle_name => $items) {
      $valid_product_variation_ids = array_merge($valid_product_variation_ids, array_keys($items));
    }

    return count($valid_product_variation_ids);
  }

  /**
   * {@inheritdoc}
   */
  public function validateReferenceableEntities(array $ids) {
    $valid_product_variation_ids = [];

    foreach ($this->getProductVariationsForRoute() as $bundle_name => $items) {
      $valid_product_variation_ids = array_merge($valid_product_variation_ids, array_keys($items));
    }

    return $valid_product_variation_ids;
  }

  /**
   * Get available variations for a product from the current route.
   *
   * The product can be loaded from a commerce route parameter (e.g.
   * /product/{commerce_product}) or from the source entity of a registration,
   * if that source entity is a product.
   *
   * @param bool $filter
   *   TRUE to filter the variations or FALSE to return them all.
   *
   * @return array
   *   A nested array of entities, the first level is keyed by the
   *   variation bundle (e.g. 'default'), which contains an array of entity
   *   labels keyed by the entity ID.
   */
  private function getProductVariationsForRoute($filter = TRUE) {
    $product = FALSE;
    if ($product = $this->routeMatch->getParameter('commerce_product')) {
      // The $product entity has been loaded from a {commerce_product}
      // parameter.
    }
    elseif ($registration = $this->routeMatch->getParameter('registration')) {
      $source_entity = $registration->getSourceEntity();
      if ($source_entity instanceof ProductInterface) {
        $product = $source_entity;
      }
    }

    if (!$product) {
      return [];
    }

    $return = [];
    $variation_storage = $this->entityTypeManager->getStorage('commerce_product_variation');

    // ProductVariationStorage::loadEnabled() is a bit better than
    // $product->getVariations() because it runs the
    // `ProductEvents::FILTER_VARIATIONS` event, which can be used to filter out
    // ineligable variations.
    $filtered_variations = $variation_storage->loadEnabled($product);

    foreach ($filtered_variations as $product_id => $variation) {
      $return[$variation->bundle()][$product_id] = $variation->label();
    }

    return $return;
  }

}
